# kyman

[![](https://img.shields.io/pypi/v/kyman.svg)](https://pypi.python.org/pypi/kyman)

[![](https://img.shields.io/travis/amauryq/kyman.svg)](https://travis-ci.com/amauryq/kyman)

[![Documentation Status](https://readthedocs.org/projects/kyman/badge/?version=latest)](https://kyman.readthedocs.io/en/latest/?badge=latest)

## Description

Python Tools for Kafka object management

* Free software: Apache Software License 2.0
* Documentation: https://kyman.readthedocs.io

## Installation

* Download and install Miniconda, after install close out the session and relaunch

```bash
wget https://repo.anaconda.com/miniconda/Miniconda3-latest-Linux-x86_64.sh && bash Miniconda3-latest-Linux-x86_64.sh
```

* Create a main folder and inside the `config` and `logs` folders, then change to the main folder

```bash
mk ~/tmp ~/tmp/config ~/tmp/log
cd cd ~/tmp
```

* Install kyman and verify the installation

```bash
pip install kyman
kyman -v
```

## Usage

### Configuration Files

* Use the following template to create the configuration file `config/kafkacmp.yml` under the main folder with the kafka clusters definition inside

```yaml
regions:
- name: dlas1
  url: https://login-dev.ultipro.com/v2/oauth2/us_internal/access_token
  client: ${IDENTITY_CLIENT}
  secret: ${IDENTITY_DLAS1_SECRET}
- name: atl
  url: https://login.ultipro.com/v2/oauth2/us_internal/access_token
  client: ${IDENTITY_CLIENT}
  secret: ${IDENTITY_ATL_SECRET}
- name: tor
  url: https://login.ultipro.ca/v2/oauth2/us_internal/access_token
  client: ${IDENTITY_CLIENT}
  secret: ${IDENTITY_TOR_SECRET}

clusters:
- alias: dlas1-mia
  url: https://topic-management-service-mia.mp-prod.dlas1.ucloud.int
  authn: dlas1
- alias: atl-loc
  url: https://topic-management-service-loc.mp-prod.atl.ucloud.int
  authn: atl
- alias: atl-agr
  url: https://topic-management-service-agr.mp-prod.atl.ucloud.int
  authn: atl
- alias: plas1-loc
  url: https://topic-management-service-loc.mp-prod.plas1.ucloud.int
  authn: atl
- alias: plas1-agr
  url: https://topic-management-service-agr.mp-prod.plas1.ucloud.int
  authn: atl
- alias: tor-loc
  url: https://topic-management-service-loc.mp-prod.tor.ucloud.int
  authn: tor
- alias: tor-agr
  url: https://topic-management-service-agr.mp-prod.tor.ucloud.int
  authn: tor
```

> _NOTE: In this kafka configuration file, you can use environment variables_

* Use the following template to create the configuration file `config/kyman.yml` under the main folder with the kafka objects definition inside
  
```yaml
clusters:
  - dlas1-mia

topics:
# FIELDS

## topicName: Name of your topic. Remember to follow the guidlines linked above.
## numPartitions: The number of partitions is how your topic is scaled through parallelism. A consumer group can support up to numParitions consumers. Default 10
## replicationFactor: The replication factor is how many nodes your topic data will be replicated to. Default 3
## configs: Configs is a map of key to value for various configs. You can find this here: https://kafka.apache.org/documentation/#topicconfigs
### cleanup.policy: This is an example configuration that allows for topic compaction

- topicName: development.internal.kyman-test.topic-one
  numPartitions: 10
  replicationFactor: 3
  configs:
    cleanup.policy: compact
    delete.retention.ms: 86400000
    segment.ms: 604800000
    min.cleanable.dirty.ratio: 0.5
  method: POST
- topicName: development.internal.kyman-test.topic-two
  numPartitions: 10
  replicationFactor: 3
  aclRequest:
    permissions:
      - READ
    principal: svc_james_test
  configs:
    cleanup.policy: compact
    delete.retention.ms: 86400000
    segment.ms: 604800000
    min.cleanable.dirty.ratio: 0.5
  method: POST
- topicName: development.internal.kyman-test.topic-three
  numPartitions: 10
  replicationFactor: 3
  configs:
    cleanup.policy: compact
    delete.retention.ms: 86400000
    segment.ms: 604800000
    min.cleanable.dirty.ratio: 0.5
  method: POST

acls:
# FIELDS

## resourceName: # The name of the topic
## permissions: # What permissions the principal needs
## principal: Your principal without the domain name (i.e. no @US.SAAS)

- resourceType: TOPIC
  permissions:
    - READ
    - WRITE
    - DESCRIBE
  principal: svc_james_test
  host: "*"
  resourceName: development.internal.kyman-test.topic-one
  method: POST
- resourceType: TOPIC
  permissions:
    - WRITE
    - DESCRIBE
  principal: svc_james_test
  host: "*"
  resourceName: development.internal.kyman-test.topic-two
  method: POST
- resourceType: TOPIC
  permissions:
    - READ
    - DESCRIBE
  principal: svc_james_test
  host: "*"
  resourceName: development.internal.kyman-test.topic-three
  method: POST
- resourceType: CONSUMER_GROUP
  permissions:
    - READ
  principal: svc_james_test
  host: "*"
  resourceName: development.kyman-test.consumer-group-one
  enableWildCard: true
  method: POST
- resourceType: TRANSACTIONAL_ID
  permissions:
    - READ
    - WRITE
    - ALL
    - DESCRIBE
    - CREATE
  principal: svc_james_test
  host: "*"
  resourceName: kyman-test.transactional-id-one
  enableWildCard: true
  method: POST
```

> _NOTE: This configuration file should be included in JIRA ticket to create the requested objects_

### Environment Variables

Configure the following environment variables to change how the program is executed

|NAME||Values|Default|
|:-|:-|:-|:-|
|DRY_RUN|Optional|True-Dry run mode/False-You can screw it up|False|
|INSECURE|Optional|True-Allow insecure calls/False-Use SSL|False|
|OUTPUT_FORMAT|Optional|(json/yaml) Output format for the log file|json|
|LOG_LEVEL|Optional|(DEBUG/INFO/WARNING/ERROR/CRITICAL)|INFO|
|KAFKA_CONFIG|Optional|Read kafka config from env var instead||
|APP_CONFIG|Optional|Read config from env var instead||


### Command line arguments

The app accept some command line arguments that can override environment variables. Use `kyman -h` to view all the available commands

## Features

The following kafka objects are supported 

* topics
* acls
  * topics
  * consumer groups
  * transactional ids
* Support multiple kafka clusters

## Development

### Quicktart

1. Clone repository: ``git clone git@github.com:UltimateSoftware/kyman.git``
2. ``cd`` into repository
3. Activate virtualenv: ``pipenv shell``
4. Fetch development dependencies ``make install-dev``

### Creating the environment from scratch

* Install `pipx` if `pipenv` and `cookiecutter` are not installed

```sh
python3 -m pip install pipx
python3 -m pipx ensurepath
```
* Install `pipenv` and `cookiecutter` using `pipx`

```sh
pipx install pipenv
pipx install cookiecutter
```
* Use `cookiecutter` to create project from the template you want

```sh
pipx run cookiecutter <cookie-cutter-template>
```

* Create a virtual environment for the project if not exist

```sh
pipenv install --dev
```

* To activate project's virtualenv, run

```sh
pipenv shell
```

* Sync dependencies from Pipenv file to setup.py

```sh
# install package
pipenv install --dev pipenv-setup
# sync setup.py with Pipfile.
## Use Pipfile and not Pipfile.lock if --pipfile is added
## Also sync dev dependencies if --dev is added
pipenv-setup sync --pipfile --dev
```
## Credits

This package was created with [`cookiecutter`](https://github.com/cookiecutter/cookiecutter) and the [`audreyfeldroy/cookiecutter-pypackage`](https://github.com/audreyfeldroy/cookiecutter-pypackage) project template.

## References

[How to set up a perfect Python project](https://sourcery.ai/blog/python-best-practices/)

[A fork of the above with a PyTest, strict flake8 checking with Travis/Tox](https://github.com/Nekroze/cookiecutter-pypackage)

[Python Best Practices Cookiecutter](https://github.com/sourcery-ai/python-best-practices-cookiecutter)
